#!/bin/bash

# Script used to run a single test.
#
# Parameters are set using config.bash script.
#

# others
case "$HOSTNAME" in
#    milth)
#        BASE=/hardmnt/milth0/sra/mover/works/HYCOMP/trunk/exp/bin/
#        ;;
    fbkclshpc*)
        # local
        CLUSTER_HOME=/fbk-ns-groups/ES/mover
        BENCH_PATH=${CLUSTER_HOME}/HYCOMP/trunk/new_exp
        BASE=${BENCH_PATH}/bin/
        ;;
    *)
        BASE=../bin
        ;;
esac;
source ${BASE}/config.bash

SET_ARCH="setarch x86_64 -R taskset -c 0 "
KEEP_TMP_FILES="y"

#------------------------------------------------------------------------------#
# usage
#
usage() {
    cat <<EOF
usage: $0 -f <input_model> 
          -meth {comp incr inter scenario bmc_hybrid} /* Method tested. */
          -m {timed, timedstamps} /* Untiming semantic. */
          [-p <property>] /* Reachability property checked. */
          [-n <prop_number>] /* Invarspec property to check */
          [-scenario <scenario_input>] /* scenario */       

          [-k] /* max bound for bmc */
          [-a] /* Avoid consecutive time actions */

          /* Options for scenario checking */
          [-invar] /* use invariant to simplify search on the scenario
                      only for "scenario" method */
          [-global] /* compile the monitor in a global automaton -
                       only when the scenario is translated into an automaton */
          [-concrete_induction] /*enable the concrete induction check */
          [-notinc] /* disable the incremental check */
          [-manual_bound] /* use the manual bounds in the search */

          /* Options for "inter" and "bmc_hybrid" method */
          [-t] /* add time variable  */
          [-step_semantics] /* use step semantics in the interleaving
                               model */
          [-smv_file_name <name>] /* just write the smv model in the specified filename */

          /* Specific options for "inter" */
          [-d] /* optimize compilation of untimed semantic, merging delta */
          [-bmc_algo] /* used invariant algorithm */
          [-i] /* check invariant incrementally  */

          /* Specific options for "bmc_hybrid" */
          [-alternate] /* alternate discrete and continuous steps */
          [-avoidloops] /* avoid to loop on the initial condition and on unsafe states */
          [-induction] /* inductive check */

          /* Options for "comp" method */
          [-use_int_hrc]  /* use hrc hierarchy of interleaving model */
          [-push_impl]  /* Push implications into conjunction */
          [-g] {use_model, app_quant} /* generalization step  */
          [-u] {monitor, refine, model} /* type of under-approx  */
          [-init_over] {true, discrete} /* type of initial over-approximation*/
          [-simp] /* use simplification */
          [-new_over] /* use neighbours over-approximations */
          [-incr] /* use incremental */
          [-flag] /* use flag */ 
          [-no_back_k] /* avoid full backtracking on k */ 
          [-exhaustive_k] /* performs an exhaustive search in each bmc step */ 
          [-new_over] /* use the new overapproximartion */

          /* Options for "incr" method  */
          [-b] /* Use BFS search (instead of DFS) */
          [-s] {local, final, delayed} /* synch type to use */
          [-c] {ufack, ufdtc, ground} /* counter type name */
          [-j] /* jump is true*/

          /* general options */
          [-vm {none, run}] /* use run tool to execute test */
          [-v n]      /* verbosity level */
          [-to n]     /* time out for -vm run (ignored if not -vm run) */
          [-mo n]     /* memory out for -vm run (ignored if not -vm run) */
EOF
    exit 1
}

#------------------------------------------------------------------------------#
# Call the compositional reachability (hy_comp_check_reachability)
#-------------------------------------------------------------------------------
check_comp() {
    local cmd=${1} 
    local model=${2}
    local mod_time=${3}
    local property=${4}
    local prop_number=${5}
    local verbosity=${6}

    local bmc_k=${7}
    local avoid_time_action=${8}

    local use_int_hrc=${9}
    local push_impl=${10}
    local gen_step_mode=${11}

    local step_semantics=${12}

    local underapp_type=${13}
    local add_time_var=${14}
    local use_simp=${15}
    local new_over=${16}

    local incr=${17}
    local flag=${18}

    local init_overapp_type=${19}
    local no_back_k=${20}
    local exhaustive_k=${21}

    local tmp=${TMPDIR}/tmp.check_comp.check.$$

    local set_bmc_cmd=""
    if [ "${bmc_k}" != "" ]; then
        set_bmc_cmd="set bmc_length ${bmc_k}"
    fi

    if [ "${property}" != "" ]; then
        property="-t \"${property}\""
    fi

    if [ "${prop_number}" != "" ]; then
        prop_number="-n ${prop_number}"
    fi

    cat >${tmp} <<EOF
set verbose_level ${verbosity}
set on_failure_script_quits

hycomp_read_model  -i ${model}
hycomp_compile_model
hycomp_compose_network -m ${mod_time} ${avoid_time_action} ${step_semantics} ${add_time_var}

hycomp_create_processes ${use_int_hrc} ${push_impl}

${set_bmc_cmd}

time
hycomp_check_reachability ${property} ${prop_number} -k ${bmc_k} ${gen_step_mode} ${underapp_type} ${use_simp} ${new_over} ${incr} ${flag} ${init_overapp_type} ${no_back_k} ${exhaustive_k}
time

quit -x
EOF

    echo "Script file is ${tmp}"
    test=`pwd`
    echo "current dir is ${test}"
    echo "cmd ${SET_ARCH} ${cmd} ${NUSMV} -pre cpp -source  ${tmp}"
    ${SET_ARCH} ${cmd} ${NUSMV} -pre cpp -source  ${tmp}

    if [ "${KEEP_TMP_FILES}X" == "X" ]; then
        rm -f ${tmp}
    fi
}

#-------------------------------------------------------------------------------
# Call one of the  monolithic standard BMC algorithm
#-------------------------------------------------------------------------------
check_inter() {
    local cmd=${1} 
    local model=${2}
    local mod_time=${3}
    local property=${4}
    local prop_number=${5}
    local verbosity=${6}

    local bmc_k=${7}
    local avoid_time_action=${8}

    local bmc_alg=${9}
    local inc=${10}
    local merge_delta=${11}
    local add_time_var=${12}
    local step_semantics=${13}
    local scenario_file=${14}

    local global=${15}

    local smv_file_name=${16} 

    if [ "${bmc_k}" != "" ]; then
        bmc_k="-k ${bmc_k}"
    fi

    if [ "${property}" != "" ]; then
        property="-p \"! (${property})\""
    fi

    if [ "${prop_number}" != "" ]; then
        prop_number="-n ${prop_number}"
    fi

    if [ "${bmc_alg}" == "falsification" ]; then
        bmc_alg="falsification"
    fi

    local tmp=${TMPDIR}/tmp.check_int.check.$$
    local tmp_int_model=${TMPDIR}/tmp.check.int.model.$$
    local build_monitor=

    if [ "${scenario_file}X" == "X" ]; then
        # we do not request to build the monitor for the scenario
        build_monitor=
    else
        # build the monitor for the scenario
        build_monitor="hycomp_compile_monitor_scenario -s ${scenario_file} ${global}"

        # fix the number of the property (we suppose that it is at the first place)
        # this is a WORKAROUND!
        property="-n 1"

        # merging delta is not feasible when building the scenario
        merge_delta=
    fi

    # build the monitor 
    cat >${tmp} <<EOF
set verbose_level ${verbosity}
set on_failure_script_quits

hycomp_read_model  -i ${model}
hycomp_compile_model
hycomp_compose_network -m ${mod_time} ${merge_delta} ${avoid_time_action} ${add_time_var} ${step_semantics}

${build_monitor}

hycomp_write_hrc_network -i -o ${tmp_int_model}

quit -x
EOF
    echo "Build the monitor for ${tmp} into ${tmp_int_model}"
    ${SET_ARCH} ${NUSMV} -pre cpp -source ${tmp}
    rm -f ${tmp}

    if [ "${smv_file_name}X" != "X" ]; then

        mv ${tmp_int_model} ${smv_file_name}
        exit 0;
    fi
    
    cat >${tmp} <<EOF
set verbose_level ${verbosity}
set on_failure_script_quits

read_model -i ${tmp_int_model}
go_msat

time
msat_check_invar_bmc -a ${bmc_alg} ${bmc_k} ${inc} ${property}
time

quit -x
EOF

    echo "Inter Script file is ${tmp}"
    ${SET_ARCH} ${cmd} ${NUSMV} -pre cpp -source ${tmp}
    if [ "${KEEP_TMP_FILES}X" == "X" ]; then
        rm -f ${tmp}
        rm -f ${tmp_int_model}
    fi
}

#-------------------------------------------------------------------------------
# Call the BMC tailored to hybrid (and timed) systems.
#-------------------------------------------------------------------------------
check_bmc_hybrid() {
    local cmd=${1} 
    local model=${2}
    local mod_time=${3}
    local property=${4}
    local prop_number=${5}
    local verbosity=${6}

    local bmc_k=${7}

    local merge_delta=${8}
    local add_time_var=${9}
    local step_semantics=${10}

    local alternate=${11}
    local avoidloops=${12}

    local scenario_file=${13}
    local global=${14}
    local induction=${15}

    # By now always use interleaving hierarchy when calling the BMC approach
    local use_int_hrc="-i"

    if [ "${bmc_k}" != "" ]; then
        bmc_k="-k ${bmc_k}"
    fi

    if [ "${property}" != "" ]; then
        property="-p \"! (${property})\""
    fi

    if [ "${prop_number}" != "" ]; then
        property="-n ${prop_number}"
    fi

    local tmp=${TMPDIR}/tmp.check_bmc_hybrid.check.$$

    if [ "${scenario_file}X" == "X" ]; then
        # we do not request to build the monitor for the scenario
        build_monitor=
    else
        # build the monitor for the scenario
        build_monitor="hycomp_compile_monitor_scenario -s ${scenario_file} ${global}"

        # fix the number of the property (we suppose that it is at the first place)
        # this is a WORKAROUND!
        property="-n 1"

        # merging delta is not feasible when building the scenario
        merge_delta=
    fi    

    cat >${tmp} <<EOF
set verbose_level ${verbosity}
set on_failure_script_quits

hycomp_read_model  -i ${model}
hycomp_compile_model
hycomp_compose_network -m ${mod_time} ${merge_delta} ${add_time_var} ${step_semantics} ${use_int_hrc}

${build_monitor}

time
hycomp_check_invar_bmc ${bmc_k} ${property} ${alternate} ${avoidloops} ${induction}
time

quit -x
EOF

    echo "Script file is ${tmp}"
    ${cmd} ${NUSMV} -pre cpp -source  ${tmp}
    if [ "${KEEP_TMP_FILES}X" == "X" ]; then
        rm -f ${tmp}
    fi
}

#-------------------------------------------------------------------------------
# Call the FORTE (Shallow synchronization) approach
#-------------------------------------------------------------------------------
check_incr() {
    local cmd=${1} 
    local model=${2}
    local mod_time=${3}
    local property=${4}
    local prop_number=${5}
    local verbosity=${6}
    local bmc_k=${7}
    local avoid_time_action=${8}


    # Incr method params
    local use_bfs=${9}
    local synch_type=${10}
    local counter_type=${11}
    local use_jump=${12}

    local tmp=${TMPDIR}/tmp.check_incr.check.$$

    local set_bmc_cmd=""
    if [ "${bmc_k}" != "" ]; then
        set_bmc_cmd="set bmc_length ${bmc_k}"
    fi

    if [ "${property}" != "" ]; then
        property="-t \"${property}\""
    fi

    if [ "${prop_number}" != "" ]; then
        prop_number="-n ${prop_number}"
    fi

    if [ "${counter_type}" != "" ]; then
        counter_type="-c ${counter_type}"
    fi

    if [ "${synch_type}" != "" ]; then
        synch_type="-s ${synch_type}"
    fi

    cat >${tmp} <<EOF

set verbose_level ${verbosity}
set on_failure_script_quits 1
${set_bmc_cmd}

hycomp_read_model  -i ${model}
hycomp_compile_model
hycomp_compose_network -m ${mod_time} ${avoid_time_action}

time
hycomp_check_reachability_incr ${use_bfs} ${property} ${prop_number} ${synch_type} ${counter_type} ${use_jump}
time

quit -x
EOF

    echo "Script file is ${tmp}"
    ${SET_ARCH} ${cmd} ${NUSMV} -pre cpp -source  ${tmp}
    if [ "${KEEP_TMP_FILES}X" == "X" ]; then
        rm -f ${tmp}
    fi
}

check_scenario() {
    local cmd=${1} 
    local model=${2}
    local mod_time=${3}
    local verbosity=${4}

    local bmc_k=${5}
    local avoid_time_action=${6}
    local scenario_file=${7}

    local use_invar=${8}
    local concrete_induction=${9}
    local notinc=${10}
    local manual_bound=${11}

    if [ "${bmc_k}" != "" ]; then
        bmc_k="-k ${bmc_k}"
    fi

    local tmp=${TMPDIR}/tmp.check_scenario.check.$$

    cat >${tmp} <<EOF
set dynamic_reorder "1"
set verbose_level ${VERBOSITY}
set on_failure_script_quits

hycomp_read_model  -i ${model}
hycomp_compile_model
hycomp_compose_network -m ${mod_time} ${merge_delta} ${add_time_var} ${step_semantics} -i

time;
hycomp_check_scenario -s ${scenario_file} ${bmc_k} ${use_invar} ${concrete_induction} ${notinc} ${manual_bound} ${avoid_time_action};
time

quit -x
EOF

    echo "CMD ${SET_ARCH} ${cmd} ${NUSMV} -pre cpp -source  ${tmp}"
    ${SET_ARCH} ${cmd} ${NUSMV} -pre cpp -source  ${tmp}
    if [ "${KEEP_TMP_FILES}X" == "X" ]; then
        rm -f ${tmp}
    fi
}


#------------------------------------------------------------------------------#
# main
#


#------------------------------------------------------------------------------#
# read parameters
#

# common properties
model=
meth=
property=
prop_number=
mod_time=
bmc_k=
avoid_time_action=

# interleaving options
merge_delta=
inc=
bmc_algo= # default is falsification
bmc_algo_default=falsification # default is falsification
add_time_var=
step_semantics=
smv_file_name=

# compositional options
use_int_hrc=
push_impl=
gen_step_mode=
underapp_type=
use_simp=
new_over=
incr=
flag=
init_overapp_type=
no_back_k=
exhaustive_k=

# incremental options
use_bfs=
synch_type=
counter_type=
use_jump=

# Options for bmc_hybrid
alternate=
avoidloops=
induction=

# specific options for scenario checking
use_invar=
global=
concrete_induction=
notinc=
manual_bound=

# bmc_hybrid
alternate=
avoidloops=


verbosity=0 # default is no verbose output 
vm=
timeout=
memout=

scenario_file=

while [ "${1}" != "" ]; do
    case "${1}" in
        -f)
            shift
            if [ "${1}" == "" ]; then
                echo "Error: No input after -f option!"
                usage
            else
                model="${1}"
                shift
            fi
            ;;
        -meth)
            shift
            case "${1}" in
                comp)
                    meth=comp
                    shift
                    ;;
                inter)
                    meth=inter
                    shift
                    ;;
                incr)
                    meth=incr
                    shift
                    ;;
                scenario)
                    meth=scenario
                    shift
                    ;;
                bmc_hybrid)
                    meth=bmc_hybrid
                    shift
                    ;;
                *)
                    echo "Error: Unknown method: ${1}!"
                    usage
                    ;;
            esac
            ;;
        # General options
        -m)
            shift
            case "${1}" in
                timed)
                    shift
                    mod_time="timed"
                    ;;
                timestamps)
                    mod_time="timestamps"
                    shift
                    ;;
                *)
                    echo "Error: ${1} is not a valid untime mode!"
                    usage
                    ;;
            esac
            ;;
        -p)
            shift
            if [ "${1}" == "" ]; then
                echo "Error: No property provided!"
                usage
            else
                if [ "${prop_number}" != "" ]; then
                    echo "Error: Cannot provide property and property number!"
                    usage
                else 
                    property="${1}"
                    shift
                fi
            fi
            ;;
        -n)
            shift
            if [ "${1}" == "" ]; then
                echo "Error: No property number provided!"
                usage
            else
                if [ "${property}" != "" ]; then
                    echo "Error: Cannot provide property and property number!"
                    usage
                else 
                    prop_number="${1}"
                    shift
                fi
            fi
            ;;
        -k)
            shift
            if [ "${1}" == "" ]; then
                echo "Error: No value for -k option!"
                usage
            else
                bmc_k="${1}"
                shift
            fi
            ;;
        -a)
            avoid_time_action="-a";
            shift
            ;;
        -t)
            add_time_var="-v";
            shift;
            ;;

# Options for interleaving method
        -d)
            merge_delta="-d";
            shift
            ;;
        -i)
            inc="-i"
            shift
            ;;
        -bmc_algo)
            shift
            case ${1} in
                classic)
                    bmc_algo=classic
                    shift
                    ;;
                een-sorensson)
                    bmc_algo=een-sorensson
                    shift
                    ;;
                falsification)
                    bmc_algo=falsification
                    shift
                    ;;
                dual)
                    bmc_algo=dual
                    shift
                    ;;
                zigzag)
                    bmc_algo=zigzag
                    shift
                    ;;
                interp_seq)
                    bmc_algo=interp_seq
                    shift
                    ;;
                *)
                    echo "Error: wrong invariant algorithm type ${1}!"
                    usage
                    ;;
                esac
            ;;
        -step_semantics)
            step_semantics="-s"
            shift
            ;;
        -smv_file_name)
            shift
            if [ "${1}" == "" ]; then
                echo "Error: No smv file name provided!"
                usage
            else
                smv_file_name="${1}"
                shift
            fi
            ;;
# Options for compositional method
        -use_int_hrc)
            use_int_hrc="-i"
            shift
            ;;
        -push_impl)
            push_impl="-p"
            shift
            ;;
        -g)
            shift
            case ${1} in
                use_model)
                    gen_step_mode="-g use_model"
                    shift
                    ;;
                app_quant)
                    gen_step_mode="-g app_quant"
                    shift
                    ;;
                *)
                    echo "Error: wrong generalization mode ${1}!"
                    usage
                    ;;
            esac
            ;;
        -u)
            shift
            case ${1} in
                monitor)
                    underapp_type="-u monitor"
                    shift
                    ;;
                refine)
                    underapp_type="-u refine"
                    shift
                    ;;
                model)
                    underapp_type="-u model"
                    shift
                    ;;
                *)
                    echo "Error: wrong underapp type ${1}!"
                    usage
                    ;;
            esac
            ;;        
        -init_over)
            shift
            case ${1} in
                true)
                    init_overapp_type="-j true"
                    shift
                    ;;
                discrete)
                    init_overapp_type="-j discrete"
                    shift
                    ;;
                *)
                    echo "Error: wrong initial overapp type ${1}!"
                    usage
                    ;;
            esac
            ;;
	-simp)
	    use_simp="-i"
	    shift
	    ;;
        -new_over)
	    new_over="-v"
	    shift
	    ;;
        -incr)
            incr="-p"
            shift
            ;;
        -flag)
            flag="-a"
            shift
            ;;
        -no_back_k)
            no_back_k="-b"
            shift
            ;;
        -exhaustive_k)
            exhaustive_k="-e"
            shift
            ;;
# Options for incremental method
        -b)
            use_bfs="-b"
            shift
            ;;
        -s)
            shift
            case ${1} in
                local)
                    synch_type=local
                    shift
                    ;;
                final)
                    synch_type=final
                    shift
                    ;;
                global)
                    synch_type=global
                    shift
                    ;;
                delayed)
                    synch_type=delayed
                    shift
                    ;;
                *)
                    echo "Error: wrong synch type ${1}!"
                    usage
                    ;;
                esac
            ;;
        -c)
            shift
            case ${1} in
                ufack)
                    counter_type=ufack
                    shift
                    ;;
                ufdtc)
                    counter_type=ufdtc
                    shift
                    ;;
                ground)
                    counter_type=ground
                    shift
                    ;;
                *)
                    echo "Error: wrong counter type ${1}!"
                    usage
                    ;;
                esac
            ;;
        -j)
            use_jump="-j"
            shift
            ;;
# scenario-based encoding
        -scenario)
            shift
            if [ "${1}" == "" ]; then                
                echo "Scenario file not provided!"
                usage
            else
                scenario_file="${1}"
                shift
            fi
            ;;
        -invar)
            use_invar="-i"
            shift
            ;;
        -concrete_induction)
            concrete_induction="-a -l -o -z concrete"
            shift
            ;;
        -notinc)
            notinc="-n"
            shift
            ;;
        -manual_bound)
            manual_bound="-m"
            shift
            ;;
# Compilation of scenario into automata
        -global)
            global="-g"
            shift
            ;;
# options for bmchybrid
        -alternate)
            alternate="-a"
            shift
            ;;
        -avoidloops)
            avoidloops="-l"
            shift
            ;;
        -induction)
            induction="-i"
            shift
            ;;
# other options
        -v)
            shift
            if [ "${1}" == "" ]; then                
                echo "Verbosity value not provided!"
                usage
            else
                verbosity="${1}"
                shift
            fi
            ;;
        -vm)
            shift
            if [ "${1}" == "none" -o \
                "${1}" == "run" ]; then
                vm="${1}"
                shift
            else
                echo "vm command not provided!"
                usage
            fi
            ;;
        -to)
            shift
            if [ "${1}" == "" ]; then
                echo "to value no provided!"
                usage
            else
                timeout="${1}"
                shift
            fi
            ;;
        -mo)
            shift
            if [ "${1}" == "" ]; then
                echo "mo value no provided!"
                usage
            else
                memout="${1}"
                shift
            fi
            ;;
        

        *)
            echo "Option ${1} not recognized!"
            usage
            ;;
    esac;
done


# needed data
if [ "${model}" == "" ]; then
    echo "NO INPUT MODEL!"
    usage
fi
if [ "${meth}" == "" ]; then
    usage
fi
if [ "${mod_time}" == "" ]; then
    echo "NO UNTIME SEMANTIC SPECIFIED!"
    usage
fi

# check method - options
if [ "${meth}" == "incr" ]; then
    if [ "${merge_delta}" != "" ]; then
        echo "Error: -d cannot be used with ${meth} method!."
        usage
    fi
fi

if [ "${meth}" != "inter" ]; then
    if [ "${inc}" != "" ] || [ "${bmc_algo}" != "" ]; then
        echo "Error: -i or -bmc_algo cannot be used with ${meth} method!."
        usage
    fi
elif [ "${meth}" != "incr" ]; then 
    if [ "${use_bfs}" != "" ] || [ "${synch_type}" != "" ] || [ "${counter_type}" != "" ] || [ "${use_jump}" != "" ]; then
        echo "Error: -b, -s, -c  or -j cannot be used with ${meth} method!."
        usage
    fi
# no special options for comp
fi

# set bmc algo default
if [ "${bmc_algo}" == "" ]; then
    bmc_algo="${bmc_algo_default}"
fi

# manage timeout and memout
if [ \( "${timeout}" != "" -o "${memout}" != "" \) -a "${vm}" != "run" ]; then
    echo "${0}: warning: values of -to, -mo will be ignored as vm != run."
fi

cmd=
case "${vm}" in
    none)
        cmd=
        ;;
    run)
        if [ "${timeout}" != "" ]; then
            timeout="--time-limit=${timeout}"
        fi
        if [ "${memout}" != "" ]; then
            memout="--space-limit=${memout}"
        fi
        cmd="${RUN} ${timeout} ${memout}"
        ;;
esac;

echo " METHOD IS ${meth}"

# SCENARIO BASED APPROACH
if [ "${meth}X" == "scenarioX" ]; then
    check_scenario "${cmd}" "${model}" "${mod_time}" "${verbosity}" "${bmc_k}" "${avoid_time_action}" "${scenario_file}" "${use_invar}" "${concrete_induction}" "${notinc}" "${manual_bound}"

# MONOLITHIC BMC
elif [ "${meth}" == "inter" ]; then
    check_inter "${cmd}" "${model}" "${mod_time}" "${property}" "${prop_number}" "${verbosity}" "${bmc_k}" "${avoid_time_action}" "${bmc_algo}" "${inc}" "${merge_delta}" "${add_time_var}" "${step_semantics}" "${scenario_file}" "${global}" "${smv_file_name}"

# MONOLITHIC BMC HYBRID 
elif [ "${meth}" == "bmc_hybrid" ]; then
    check_bmc_hybrid "${cmd}" "${model}" "${mod_time}" "${property}" "${prop_number}" "${verbosity}" "${bmc_k}" "${merge_delta}" "${add_time_var}" "${step_semantics}" "${alternate}" "${avoidloops}" "${scenario_file}" "${global}" "${induction}"

# FORTE 2010 - SHALLOW SYNCHRONIZATION APPROACH
elif [ "${meth}" == "incr" ]; then
    check_incr "${cmd}" "${model}" "${mod_time}" "${property}" "${prop_number}" "${verbosity}" "${bmc_k}" "${avoid_time_action}" "${use_bfs}" "${synch_type}" "${counter_type}" "${use_jump}"

# FMCAD09 (tentative) -COMPOSITIONAL APPROACH
elif [ "${meth}" == "comp" ]; then
    check_comp "${cmd}" "${model}" "${mod_time}" "${property}" "${prop_number}" "${verbosity}" "${bmc_k}" "${avoid_time_action}" "${use_int_hrc}" "${push_impl}" "${gen_step_mode}" "${step_semantics}" "${underapp_type}" "${add_time_var}" "${use_simp}" "${new_over}" "${incr}" "${flag}" "${init_overapp_type}" "${no_back_k}" "${exhaustive_k}"
fi

