#!/bin/bash

TOTTIMEOUT=900
MEMOUT=4094
VERBOSITY=0

echo "Test run on $HOSTNAME on `date`"

case "$HOSTNAME" in
    fbkclshpc*)
        CLUSTER_HOME=/fbk-ns-groups/ES/mover
        BENCH_PATH=${CLUSTER_HOME}/HYCOMP/trunk/new_exp
        PDIR=${BENCH_PATH}
        ;;
    *)
        PDIR=.
        ;;
esac;

. ${PDIR}/bin/config.bash

################################################################################
# Hash table functions

# put values in hash table
# $1: hash map name
# $2: key
# $3: value
hput () {
  eval "$1""$2"='$3'
}

# retrieve values from hash tables and put the value in the passed parameter
# $1: hash map name
# $2: key
# $3: return value
hget () {
    eval $3='${'"$1$2"'#hash}'
#    eval $3='${$1'"$2"'#hash}'
}

#------------------------------------------------------------------------------#
# save the msat trace
#
save_trace() {
  local log_name=${1}
  local methswitch=${2}

  log_name="${log_name%.*}_msat_trace.txt"

  if [ "${methswitch}X" = "-meth interX" ]; then
      trace_file_name="/tmp/msat_bmc_trace.smt2"
  else
      trace_file_name="/tmp/tracing.txt"
  fi

  if [ -e ${trace_file_name} ]; then
      mv ${trace_file_name} ${log_name}
  fi
}


# input parameters 
# 1. directory which contains models
# 2. name of the method used to run the test
models_dirs=$1
meths=$2

# read config file

pushd .
PDIR=`cd ${PDIR}; pwd`

cd ${PDIR}
cd ${models_dirs}

config_file_name="config.test"
if [ -e "${config_file_name}" ]; then
    K_inter=`cat config.test | grep "K_inter"`
    K_inter=${K_inter##*=}

    K_step=`cat config.test | grep "K_step"`
    K_step=${K_step##*=}

    K_scenario=`cat config.test | grep "K_scenario"`
    K_scenario=${K_scenario##*=}


    sort_params=`cat config.test | grep "sort_params"`
    sort_params=${sort_params##*=}
else
    echo "Configuration file not found!"
    echo "Skipping ${dir}..."        
    exit 1
fi


# Set standard option (now no loops on options is performed)
# exported to env in order to choose options given method name.
#
# The pattern is <method_name>_opt


# OPTIONS for MONOLITHIC BMC
export inter_step_sem_opt="-k ${K_step} -d -i -t -m timed -step_semantics"
export inter_step_sem_local_time_opt="-k ${K_step} -i -m timestamps -step_semantics"
export inter_step_sem_global_opt="-k ${K_step} -i -m timestamps -step_semantics -global"

# Options for the BMC HYBRID monolithic approach
# By now we both enable the alternation of local/global steps and we avoid loops on the initial condition
export bmc_hybrid_step_sem_opt="-k ${K_step} -t -m timed -step_semantics -avoidloops -alternate"
export bmc_hybrid_step_sem_local_time_opt="-k ${K_step} -m timestamps -step_semantics -avoidloops -alternate"
export bmc_hybrid_step_sem_global_opt="-k ${K_step} -m timestamps -step_semantics -global -avoidloops -alternate"


# OPTIONS FOR THE SCENARIO-BASED APPROACH
export scenario_step_sem_opt="-k ${K_scenario} -m timestamps -step_semantics"
export scenario_step_sem_invar_opt="-k ${K_scenario} -m timestamps -step_semantics -invar"
# non-incremental approaches
export scenario_ni_step_sem_opt="-k ${K_scenario} -m timestamps -step_semantics -notinc"
export scenario_ni_step_sem_invar_opt="-k ${K_scenario} -m timestamps -step_semantics -invar -notinc"
# alternation of timed and discrete steps
export scenario_step_sem_avoid2timed_opt="-k ${K_scenario} -m timestamps -step_semantics -a"
export scenario_step_sem_invar_avoid2timed_opt="-k ${K_scenario} -m timestamps -step_semantics -invar -a"
# non-incremental approaches
export scenario_ni_step_sem_avoid2timed_opt="-k ${K_scenario} -m timestamps -step_semantics -notinc -a"
export scenario_ni_step_sem_invar_avoid2timed_opt="-k ${K_scenario} -m timestamps -step_semantics -invar -notinc -a"
# bounded approaches
export scenario_bounded_step_sem_opt="-k ${K_scenario} -m timestamps -step_semantics -manual_bound"
export scenario_bounded_step_sem_invar_opt="-k ${K_scenario} -m timestamps -step_semantics -invar -manual_bound"
# non-incremental approaches
export scenario_bounded_ni_step_sem_opt="-k ${K_scenario} -m timestamps -step_semantics -notinc -manual_bound"
export scenario_bounded_ni_step_sem_invar_opt="-k ${K_scenario} -m timestamps -step_semantics -invar -notinc -manual_bound"



# k-induction
export bmc_hybrid_ind_step_sem_local_time_opt="-k ${K_step} -m timestamps -step_semantics -alternate -induction"
# induction method
export scenario_ind_step_sem_opt="-k ${K_scenario} -m timestamps -step_semantics -concrete_induction"


startdate=`date`
inserted_value=

  # reset method hash map
for key in ${inserted_keys}; do
    hput meth_hash "${key}" ""
done
inserted_keys=""

for model_file in `find . -name "*hydi"  | sort -t '_' ${sort_params}`; do
    model_name=${model_file##*/} # get rid of path
    model_name=${model_name%.*} # get rid of extension          
    i=`echo "${model_name}" | sed "s:[a-z\_]*\_\([0-9]*\):\1:"`

    # hack for bmw
    is_bmw=`echo ${model_name} | grep bmwlin`
    if [ "${is_bmw}X" != "X" ]; then
        search_str="bmwlin_*[0-9].scen"
    # hack for audio - can be generalized if we do not match a number
    elif [ "${model_name}X" == "audioX" ]; then
        search_str="audio_[0-9]*.scen"
    else
        search_str="*_${i}_*[0-9].scen"
    fi

    echo "${model_name} i is ${i}: find ${search_str}. -name ${search_str}"
    for scenario_file in `find . -name "${search_str}"`; do
        scenario_name=${scenario_file%.*}
        scenario_name=${scenario_name##*/}


        double_number=`echo "${scenario_name}" | sed "s:_[0-9]*_[0-9]*.:|fake_string|:g" | grep "|fake_string|"`
        if [ "${double_number}X" != "X" ]; then
            # we matched the pattern, 2 numbers 
            num_cycles=`echo "${scenario_name%.*}" | sed "s:[a-z\_]*\_[0-9]*\_\([0-9]*\):\1:"`
        else
            # we did not matched anything, one only pattern
	    num_cycles=`echo "${scenario_name%.*}" | sed "s:[a-z\_]*\_\([0-9]*\):\1:"`
	fi

        echo "${model_file} ${scenario_file} ${num_cycles}"

        for meth in ${meths}; do


            key="${meth}_${num_cycles}"

            # we do not manage partitioning! 
            #
            # # Check if benchmarks with potrait partitioning are tested
            # # it is not empty if file is filename_number_number.hydi
            # has_partitioning=`echo ${model_file}  | egrep "[a-z]*_[0-9]*_[0-9]*"`
            # if [ "${has_partitioning}X" != "X" ]; then
            #   # get the number of parition
            #     part_numb=`echo "${model_name}" | sed "s:[a-z\_]*\_[0-9]*\_\([0-9]*\):\1:"`
            #     key="${meth}_${part_numb}"
            # else 
            #     key="${meth}_0"
            # fi

            # query hash to see wether stop search
            hget meth_hash "${key}" stop_current_meth

            if [ "${stop_current_meth}X" == "X" ]; then
                # keep to search on method
                opt=
                if [ "${ONLY_SMV_TRANSLATION}X" != "yesX" ]; then
                    log=${model_name}_${num_cycles}.${meth}.txt
                else
                    log=${model_name}_${num_cycles}.${meth}.translation.txt
                fi

                echo -n -e "Doing ${log} ..."

                # do not overwrite existing log files
                if [ -e ${log} ] &&
                    [ "${ONLY_SMV_TRANSLATION}X" != "yesX" ]; then
                    echo "...file already exists"
                else
                    if [ "${ONLY_SMV_TRANSLATION}X" != "yesX" ]; then
                        tmp_log=${TMPDIR}/${model_name}_${num_cycles}.${meth}.txt
                        smv_file_name=""
                    else
                        tmp_log=${TMPDIR}/${model_name}_${num_cycles}.${meth}.translation.txt
                        smv_file_name="-smv_file_name ${model_name}_${num_cycles}.${meth}.smv"
                    fi

                    checkcmd="${CHECK}"

                    has_incr=`echo ${meth} | grep inc` 
                    if [ "${has_incr}L" != "L" ]; then
                        # method is incremental
                        methswitch=incr
                    fi

                    has_inter=`echo ${meth} | grep int` 
                    if [ "${has_inter}L" != "L" ]; then
                        # method is interleaving
                        methswitch=inter
                    fi

                    has_comp=`echo ${meth} | grep comp`
                    if [ "${has_comp}L" != "L" ]; then
                        # method is compositional
                        methswitch=comp
                    fi

                    has_scenario=`echo ${meth} | grep scenario`
                    if [ "${has_scenario}L" != "L" ]; then
                        # method is compositional
                        methswitch=scenario
                    fi

                    has_bmc_hybrid=`echo ${meth} | grep "bmc_hybrid"`
                    if [ "${has_bmc_hybrid}L" != "L" ]; then
                        # method is bmc_hybrid
                        methswitch=bmc_hybrid
                    fi

                    methswitch="-meth ${methswitch}"

                    # This works since option variables are exported into environment
                    opt=`env | grep ${meth}_opt | sed "s:.*=::"`

                    echo "Options ${opt}." >>${tmp_log}

                    checkcmd="${checkcmd} -f ${model_file} ${methswitch} ${opt} -v ${VERBOSITY} -vm run -to ${TOTTIMEOUT} -mo ${MEMOUT} -n 0 -scenario ${scenario_file} ${smv_file_name}"

                    echo "${checkcmd}" >>${tmp_log}
                    ${checkcmd} >>${tmp_log} 2>&1


                    ris=`cat ${tmp_log} | grep "out of time"`
                    if [ "${ris}X" != "X" ]; then
                        echo "${log} went out of time!";
                        echo "Inserting key ${key}"
                        hput meth_hash "${key}" "present"
                        inserted_keys="${inserted_keys} ${key}"
                    else
                        if [ "${HYCOMP_SAVE_MSAT_TRACES}X" = "yesX" ]; then
                            save_trace "${log}" "${methswitch}"
                        fi
                    fi

                    # move tmp log to real log
		    mv ${tmp_log} ${log}

                    echo " done"
                fi # end log exist if
            else 
                # method skipped
                echo "SKIPPED SEARCH ON: ${model_name}.${meth}.txt "
                stop_current_meth=""
            fi # end of if keep search on current method
        done # end loop in methods
    done # end loop on scenarios

done # end loops on models

# send notification mail
enddate=`date`
subject="ENDED ${models_dirs} - ${meths} ENDED at ${enddate}"
msg="Task ${models_dirs} - ${meths} started at ${startdate} and ended at ${enddate}"
mailsend=`echo "${msg}" | mutt -s "${subject}" ${NOTIFYTO}`


popd

